// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneItemComponent.hpp"
#include "scene/components/display/MeshRenderer.hpp"
#include "scene/components/display/MeshHost.hpp"
#include "display/mesh/QuadMesh.hpp"
#include "display/Tileset.hpp"

#define TILED_SPRITE_FLIP_Y FLAG_DEFINE(0)
#define TILED_SPRITE_FLIP_X FLAG_DEFINE(1)

namespace Dawn {
  class TiledSprite : public SceneItemComponent {
    protected:
      MeshRenderer *renderer = nullptr;
      MeshHost *host = nullptr;
      Tileset *tileset = nullptr;
      flag_t flipState = TILED_SPRITE_FLIP_Y;
      int32_t tileIndex = -1;
      glm::vec2 xy0 = glm::vec2(0, 0);
      glm::vec2 xy1 = glm::vec2(1, 1);

      glm::vec2 getUV0();
      glm::vec2 getUV1();

    public:
      TiledSprite(SceneItem *item);

      void setTileset(Tileset *tileset);
      void setTilesetAndSize(TilesetGrid *gridTileset, glm::vec2 center);
      void setTilesetAndSize(TilesetGrid *gridTileset);
      void setTile(int32_t tile);
      void setFlippedState(flag_t flippedState);
      void setSize(glm::vec2 size, glm::vec2 center);
      void setSize(glm::vec2 size);

      std::vector<SceneItemComponent*> getDependencies() override;
      void onStart() override;
  };
}